// primitive graphics for Hello World PSP

#include <pspkernel.h>
#include <pspctrl.h>
#include <psppower.h>
#include <string.h>
#include <stdio.h>
#include "pg.h"
#include "fonts/fontNaga10.c"
#include "fonts/font_ega.c_" // Classic EGA font - mixed case

int analog_x=0, analog_y=0;

extern u16 _plugbmp[];

int HomeVisible=0;

//system call

extern u8 GREY32[32];

//variables
u8 *pg_vramtop=(u8*)0x04000000;
long pg_screenmode;
long pg_showframe;
long pg_drawframe;



void pgWaitVn(u32 count)
{
	for (; count>0; --count) {
		sceDisplayWaitVblankStart();
	}
}


void pgWaitV()
{
	sceDisplayWaitVblankStart();
}


u8 *pgGetVramAddr(u32 x,u32 y)
{
	return pg_vramtop+(pg_drawframe?FRAMESIZE:0)+x*PIXELSIZE*2+y*LINESIZE*2+0x40000000;
}


void pgInit()
{
	sceDisplaySetMode(0,SCREEN_WIDTH,SCREEN_HEIGHT);
	pgScreenFrame(0,0);
}


void pgPrint(u32 x,u32 y,u32 color,const char *str)
{
	while (*str!=0 && x<CMAX_X && y<CMAX_Y) {
		pgPutChar(x*8,y*8,color,0,*str,1,0,1);
		str++;
		x++;
		if (x>=CMAX_X) {
			x=0;
			y++;
		}
	}
}

void pgPrint2(u32 x,u32 y,u32 color,const char *str)
{
	while (*str!=0 && x<CMAX2_X && y<CMAX2_Y) {
		pgPutChar(x*16,y*16,color,0,*str,1,0,2);
		str++;
		x++;
		if (x>=CMAX2_X) {
			x=0;
			y++;
		}
	}
}


void pgPrint4(u32 x,u32 y,u32 color,const char *str)
{
	while (*str!=0 && x<CMAX4_X && y<CMAX4_Y) {
		pgPutChar(x*32,y*32,color,0,*str,1,0,4);
		str++;
		x++;
		if (x>=CMAX4_X) {
			x=0;
			y++;
		}
	}
}

void pgDrawFrame(unsigned long x1, unsigned long y1, unsigned long x2, unsigned long y2, unsigned long color)
{
        unsigned char *vptr0;           //pointer to vram
        unsigned long i;

        vptr0=(unsigned char *)pgGetVramAddr(0,0);
        for(i=x1; i<=x2; i++){
                ((unsigned short *)vptr0)[i*PIXELSIZE + y1*LINESIZE] = color;
                ((unsigned short *)vptr0)[i*PIXELSIZE + y2*LINESIZE] = color;
        }
        for(i=y1; i<=y2; i++){
                ((unsigned short *)vptr0)[x1*PIXELSIZE + i*LINESIZE] = color;
                ((unsigned short *)vptr0)[x2*PIXELSIZE + i*LINESIZE] = color;
        }
}

void pgFillBox(unsigned long x1, unsigned long y1, unsigned long x2, unsigned long y2, unsigned long color)
{
	unsigned char *vptr0;		//pointer to vram
	unsigned long i, j;

	vptr0=(unsigned char *)pgGetVramAddr(0,0);
	for(i=y1; i<=y2; i++){
		for(j=x1; j<=x2; j++){
			((unsigned short *)vptr0)[j*PIXELSIZE + i*LINESIZE] = color;
		}
	}
}

void pgGradientBox(u32 x1, u32 y1, u32 x2, u32 y2, u8 r1, u8 g1, u8 b1, u8 r2, u8 g2, u8 b2, u16 w)
{
	unsigned char *vptr0;		//pointer to vram
	unsigned long i, j, x, z, c;

	vptr0=(unsigned char *)pgGetVramAddr(0,0);
	for(i=y1; i<=y2; i++){
		for(j=x1; j<=x2; j++){
		  z = j-x1;
		  x = x2-j;
		  if(x<w) {
                  c = RGB(((r1*z)/w), ((g1*z)/w), ((b1*z)/w)) & 0xfff;
                  c+= RGB(((r2*x)/w), ((g2*x)/w), ((b2*x)/w));
                  } else if(z<w) {
                  c = RGB(((r1*x)/w), ((g1*x)/w), ((b1*x)/w)) & 0xfff;
                  c+= RGB(((r2*z)/w), ((g2*z)/w), ((b2*z)/w));
                  } else {
                    c = RGB(r2,g2,b2);
                  }
			((unsigned short *)vptr0)[j*PIXELSIZE + i*LINESIZE] = c;
		}
	}
}

void pgFillvram(u32 color)
{
	u8 *vptr0;		//pointer to vram
	u32 i;

	vptr0=pgGetVramAddr(0,0);
	for (i=0; i<FRAMESIZE/2; i++) {
		*(u16 *)vptr0=color;
		vptr0+=PIXELSIZE*2;
	}
}

static u16 fadebuf[480*272];
void pgSetupFade() {
  int i;
  for(i=0; i<272; i++)
    memcpy(&fadebuf[i*480], pgGetVramAddr(0,i), 480*2);
}

void pgFadevram(u32 p, u16 *buf)
{
	u8 *vptr0;		//pointer to vram
	u32 x,y, r,g,b, c;
	if(!buf) buf = fadebuf;

	for (y=0; y<272; y++) {
	  vptr0=pgGetVramAddr(0,y);
 	  for (x=0; x<480; x++) {
            c = buf[y*480+x];
            b = (((c>>7) & 0xf8)*p)/100;
            g = (((c>>2) & 0xf8)*p)/100;
            r = (((c<<3) & 0xf8)*p)/100;
            *(u16 *)vptr0=RGB(r,g,b);
            vptr0+=PIXELSIZE*2;
          }
	}
}

void pgStipplevram(u32 color)
{
	u8 *vptr0;		//pointer to vram
	u32 x,y;

	for (y=0; y<272; y++) {
	  vptr0=pgGetVramAddr((y&1),y);
 	  for (x=(y&1); x<480; x+=2) {
		*(u16 *)vptr0=color;
		vptr0+=PIXELSIZE*4;
	  }
        }
}


void pgBitBltR(bltrect *sr, u16 dx, u16 dy) {
	u8 *vptr0, *vptr;
	u16 x, y;

	if(sr) {
		u16 sw = sr->w, sh = sr->h;
		u16 *sb = sr->b;

		vptr0=pgGetVramAddr(dx, dy);
		for(y=0; y<sh; y++) {
			sb = sr->b + (sr->s * (y + sr->y)) + sr->x;
			vptr=vptr0;
			for(x=0; x<sw; x++) {
				*(u16 *)vptr = *sb++;
				vptr+=PIXELSIZE*2;
			}
			vptr0+=LINESIZE*2;
		}
	}
}

void pgBitBltD(bltrect *ds, bltrect *sr) {
	u16 x, y;

	if(sr && ds) {
		u16 sw = sr->w, sh = sr->h;
		u16 *sb, *db;

		for(y=0; y<sh; y++) {
			sb = sr->b + (sr->s * (y + sr->y)) + sr->x;
			db = ds->b + (ds->s * (y + ds->y)) + ds->x;
			for(x=0; x<sw; x++) {
				*db++ = *sb++;
			}
		}
	}
}

void pgBitBltRT(bltrect *sr, u16 dx, u16 dy, u16 t) {
	u8 *vptr0, *vptr;
	u16 x, y;

	if(sr) {
		u16 sw = sr->w, sh = sr->h;
		u16 *sb = sr->b;

		vptr0=pgGetVramAddr(dx, dy);
		for(y=0; y<sh; y++, vptr0+=LINESIZE*2) {
			sb = sr->b + (sr->s * y);
			vptr=vptr0;
			for(x=0; x<sw; x++, sb++, vptr+=2) {
			        if(*sb != t)
				  *(u16 *)vptr = *sb;
			}
		}
	}
}

void pgBitBlt(u32 x,u32 y,u32 w,u32 h,u32 mag,const u16 *d)
{
	u8 *vptr0;		//pointer to vram
	u8 *vptr;		//pointer to vram
	u32 xx,yy,mx,my;
	const u16 *dd;
	
	vptr0=pgGetVramAddr(x,y);
	for (yy=0; yy<h; yy++) {
		for (my=0; my<mag; my++) {
			vptr=vptr0;
			dd=d;
			for (xx=0; xx<w; xx++) {
				for (mx=0; mx<mag; mx++) {
					*(u16 *)vptr=*dd;
					vptr+=PIXELSIZE*2;
				}
				dd++;
			}
			vptr0+=LINESIZE*2;
		}
		d+=w;
	}
	
}


void pgPutChar(u32 x,u32 y,u32 color,u32 bgcolor,u8 ch,char drawfg,char drawbg,char mag)
{
	u8 *vptr0;		//pointer to vram
	u8 *vptr;		//pointer to vram
	const u8 *cfont;		//pointer to font
	u32 cx,cy;
	u32 b;
	char mx,my;

	// if (ch>255) return;
	cfont=font+ch*8;
	vptr0=pgGetVramAddr(x,y);
	for (cy=0; cy<8; cy++) {
		for (my=0; my<mag; my++) {
			vptr=vptr0;
			b=0x80;
			for (cx=0; cx<8; cx++) {
				for (mx=0; mx<mag; mx++) {
					if ((*cfont&b)!=0) {
						if (drawfg) *(u16 *)vptr=color;
					} else {
						if (drawbg) *(u16 *)vptr=bgcolor;
					}
					vptr+=PIXELSIZE*2;
				}
				b=b>>1;
			}
			vptr0+=LINESIZE*2;
		}
		cfont++;
	}
}


void pgScreenFrame(long mode,long frame)
{
	pg_screenmode=mode;
	frame=(frame?1:0);
	pg_showframe=frame;
	if (mode==0) {
		//screen off
		pg_drawframe=frame;
		sceDisplaySetFrameBuf(0,0,0,1);
	} else if (mode==1) {
		//show/draw same
		pg_drawframe=frame;
		sceDisplaySetFrameBuf((char*)pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,1);
	} else if (mode==2) {
		//show/draw different
		pg_drawframe=(frame?0:1);
		sceDisplaySetFrameBuf((char*)pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,1);
	}
}


void pgScreenFlip()
{
	pg_showframe=(pg_showframe?0:1);
	pg_drawframe=(pg_drawframe?0:1);
	sceDisplaySetFrameBuf((char*)pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,0);
}


void pgScreenFlipV()
{
	pgWaitV();
	pgScreenFlip();
}

// by kwn
void Draw_Char_Hankaku(int x,int y,const unsigned char c,int col) {
	unsigned short *vr;
	unsigned char  *fnt;
	unsigned char  pt;
	unsigned char ch;
	int x1,y1;

	ch = c;

	// mapping
	if (ch<0x20)
		ch = 0;
	else if (ch<0x80)
		ch -= 0x20;
	else if (ch<0xa0)
		ch = 0;
	else
		ch -= 0x40;

	fnt = (unsigned char *)&hankaku_font10[ch*10];

	// draw
	vr = (unsigned short *)pgGetVramAddr(x,y);
	for(y1=0;y1<10;y1++) {
		pt = *fnt++;
		for(x1=0;x1<5;x1++) {
			if (pt & 1)
				*vr = col;
			vr++;
			pt = pt >> 1;
		}
		vr += LINESIZE-5;
	}
}

// by kwn
void Draw_Char_Hankaku2(int x,int y,const unsigned char c,int col) {
	unsigned short *vr;
	unsigned char  *fnt;
	unsigned char  pt;
	unsigned char ch;
	int x1,y1, nx, ny;

	ch = c;

	// mapping
	if (ch<0x20)
		ch = 0;
	else if (ch<0x80)
		ch -= 0x20;
	else if (ch<0xa0)
		ch = 0;
	else
		ch -= 0x40;

	fnt = (unsigned char *)&hankaku_font10[ch*10];

	// draw
	vr = (unsigned short *)pgGetVramAddr(x,y);
	for(y1=0;y1<10;y1++) {
		for(ny=0; ny<2; ny++) {
		  pt = *fnt;
  		  for(x1=0;x1<5;x1++) {
		        for(nx=0; nx<2; nx++) {
  			  if (pt & 1)
				*vr = col;
			  vr++;
                        }
			pt = pt >> 1;
                  }
                  vr += LINESIZE-10;
		}
		fnt++;
	}
}

// by kwn
void Draw_Char_Zenkaku(int x,int y,const unsigned char u,unsigned char d,int col) {
	// ELISA100.FNTɑ݂Ȃ
	const unsigned short font404[] = {
		0xA2AF, 11,
		0xA2C2, 8,
		0xA2D1, 11,
		0xA2EB, 7,
		0xA2FA, 4,
		0xA3A1, 15,
		0xA3BA, 7,
		0xA3DB, 6,
		0xA3FB, 4,
		0xA4F4, 11,
		0xA5F7, 8,
		0xA6B9, 8,
		0xA6D9, 38,
		0xA7C2, 15,
		0xA7F2, 13,
		0xA8C1, 720,
		0xCFD4, 43,
		0xF4A5, 1030,
		0,0
	};
	unsigned short *vr;
	unsigned short *fnt;
	unsigned short pt;
	int x1,y1;

	unsigned long n;
	unsigned short code;
	int j;

	// SJISR[h̐
	code = u;
	code = (code<<8) + d;

	// SJISEUCɕϊ
	if(code >= 0xE000) code-=0x4000;
	code = ((((code>>8)&0xFF)-0x81)<<9) + (code&0x00FF);
	if((code & 0x00FF) >= 0x80) code--;
	if((code & 0x00FF) >= 0x9E) code+=0x62;
	else code-=0x40;
	code += 0x2121 + 0x8080;

	// EUCbtHg̔ԍ𐶐
	n = (((code>>8)&0xFF)-0xA1)*(0xFF-0xA1)
		+ (code&0xFF)-0xA1;
	j=0;
	while(font404[j]) {
		if(code >= font404[j]) {
			if(code <= font404[j]+font404[j+1]-1) {
				n = -1;
				break;
			} else {
				n-=font404[j+1];
			}
		}
		j+=2;
	}
	fnt = (unsigned short *)&zenkaku_font10[n*10];

	// draw
	vr = (unsigned short *)pgGetVramAddr(x,y);
	for(y1=0;y1<10;y1++) {
		pt = *fnt++;
		for(x1=0;x1<10;x1++) {
			if (pt & 1)
				*vr = col;
			vr++;
			pt = pt >> 1;
		}
		vr += LINESIZE-10;
	}
}

// by kwn
void mh_print(int x,int y,const char *msg,int col) {
        int xx = x;
        unsigned char ch = 0,bef = 0, dbl=0, *str=(unsigned char*)msg;

        while(*str != 0) {
                ch = *str++;
                if (bef!=0) {
                        Draw_Char_Zenkaku(x,y,bef,ch,col);
                        x+=10;
                        bef=0;
                } else {
                        if (((ch>=0x80) && (ch<0xa0)) || (ch>=0xe0)) {
                                bef = ch;
                        } else {
                                if (ch=='\n'){
                                        x=xx; 
                                        y+=10;
                                } else if(ch=='\t') {
                                        dbl = !dbl;
                                } else if(dbl) {
                                        pgPutChar(x,y,col,0,ch,1,0,2);
                                        x+=16;
                                } else {
                                        Draw_Char_Hankaku(x,y,ch,col);
                                        x+=5;
                                }
                        }
                }
                if (x>=480) break;
        }
}


u32 new_pad;
u32 old_pad;
u32 now_pad;
ctrl_data_t paddata;

void HomeScreen();

void readpad(void)
{
        static int n=0;
        ctrl_data_t paddata;

        sceCtrlSetSamplingCycle(0);
        sceCtrlSetSamplingMode(1);
        sceCtrlReadBufferPositive(&paddata, 1);
        // kmg
        // Analog pad state
        analog_x = paddata.analog_x;
        analog_y = paddata.analog_y;

#if 0        
        if (paddata.analog_y == 0xff) paddata.buttons=CTRL_DOWN;
        if (paddata.analog_y == 0x00) paddata.buttons=CTRL_UP;  
        if (paddata.analog_x == 0x00) paddata.buttons=CTRL_LEFT;
        if (paddata.analog_x == 0xff) paddata.buttons=CTRL_RIGHT;
#endif

        now_pad = paddata.buttons;
        new_pad = now_pad & ~old_pad;
        if(old_pad==now_pad){
                n++;
                if(n>=25){
                        new_pad=now_pad;
                        n = 20;
                }
        }else{
                n=0;
                old_pad = now_pad;
        }
        
        if((new_pad & CTRL_HOME) && !HomeVisible)
          HomeScreen();
}

extern int bQuit;
void HomeScreen() {
  int HomeSel=0;
  int x1=170, x2=480-226, c, f=0, fadedone=0;
  HomeVisible=1;
  pgSetupFade();
  for(;;) {
    if(!fadedone) {
      pgFadevram(80-f*20, 0);
      pgFillBox(0, 271-16, 479, 271, BLACK);
      if(f==3) fadedone=1;
    }

    readpad();

    if(new_pad & (CTRL_LEFT | CTRL_RIGHT)) HomeSel = !HomeSel;
    if(new_pad & CTRL_HOME) break;
    if(new_pad & (CTRL_CROSS | CTRL_CIRCLE)) { bQuit = HomeSel; break; }
    if(new_pad & (CTRL_SQUARE | CTRL_TRIANGLE)) break;

    mh_print(240-63, 271-15, " / ~ Enter   /  Back", BLACK);
    mh_print(240-64, 271-16, " / ~ Enter   /  Back", WHITE);

    pgGradientBox(52, 90, 479-52, 91, 0,0,0, 255,255,255, 48);
    pgGradientBox(56, 92, 479-56, 152, 0,0,0, 64,64,64, 16);
    pgGradientBox(52, 153, 479-52, 154, 0,0,0, 255,255,255, 48);

    mh_print(81,97,"\tDo you want to quit?", BLACK);
    mh_print(80,96,"\tDo you want to quit?", WHITE);

    f++;
    c = RGB(GREY32[(f & 0x1f)], GREY32[(f & 0x1f)], GREY32[(f & 0x1f)]);

    if(HomeSel) {
      pgFillBox(x1, 122, x1+56, 146, c);
    } else {
      pgFillBox(x2, 122, x2+56, 146, c);
    }
    mh_print(x1+6, 129, "\tYES", BLACK);
    mh_print(x1+5, 128, "\tYES", WHITE);
    mh_print(x2+14, 129, "\tNO", BLACK);
    mh_print(x2+13, 128, "\tNO", WHITE);

    pgBattery(479-64, 4);

    pgScreenFlipV();
  }
  readpad();
  HomeVisible=0;
}

void pgBattery(int x, int y) {
  int l,w,r, bp, c;

  bp = scePowerGetBatteryLifePercent();

  l = x+3;
  w = 48;
  r = l+w+2;
  pgFillBox(x, y+8, l, y+16, WHITE);
  pgFillBox(l, y, r+3, y+24, WHITE);
  pgDrawFrame(l+1, y, r+2, y+23, WHITE);
  pgFillBox(l+2, y+2, r+1, y+22, BLACK);

  if(bp>80) c = WHITE;
  if(bp>60) c = GREEN;
  else if(bp>40) c = YELLOW;
  else if(bp>20) c = ORANGE;
  else c = RED;
  pgFillBox(r - ((w*bp)/100), y+4, r, y+20, c);

  {
    char battmp[32];
    sprintf(battmp, "\t%i", bp);
    w+=x-strlen(battmp)*8;
    mh_print(w,10, battmp, BLACK);
    mh_print(w,9, battmp, WHITE);
  }

  if(scePowerIsPowerOnline() && _plugbmp) {
    bltrect br = { 0, 0, 24, 24, 24, _plugbmp };
    pgBitBltRT(&br, x-32, y, BLACK);
  }
}
